if _G.MutatorHelper then
    _G.MutatorHelper = _G.MutatorHelper
    return
end

--[[
    Main table

    Do not touch it!
]]
_G.MutatorHelper =
{
    ModPath = ModPath,
    LocPath = ModPath .. "loc/",
    SavePath = SavePath .. "MutatorHelper.json",
    options = {},
    CustomMutators = {},
    CustomMutatorsLanguage = {},
    CustomMutatorsLanguageDefer = {},
    CustomCategories = { { name = "custom_mutators", count = 0 } },
    CustomCategoryPos = { custom_mutators = 1 },
    CustomCategoryLastPos = 2,
    CustomCategoriesLanguageFile = {}
}
local Language =
{
    [2] = "english",
    [3] = "french",
    [4] = "german",
    [5] = "italian",
    [6] = "russian",
    [7] = "swedish",
    [8] = "spanish",
    [9] = "dutch",
    [10] = "thai",
    [11] = "schinese",
    [12] = "tchinese",
    [13] = "japanese",
    [14] = "portuguese-br",
    [15] = "czech"
}
local cached_language
local Errors =
{
    [0] = "Unspecified error encountered.",
    [1] = "Passed mutator is nil.",
    [2] = "Mutator path for %s is not a string.",
    [3] = "Texture path for %s is nil.",
    [4] = "Category name is nil.",
    [5] = "Category name is not a string.",
    [6] = "Localization is nil.",
    [7] = "Localization does not have required fields.",
    [8] = "Category %s with this name already exists.",
    [9] = "Texture path is nil.",
    [10] = "Texture is not a string.",
    [11] = "ModPath is nil.",
    [12] = "ModPath is not a string.",
    [13] = "ModPath for localization is nil.",
    [14] = "ModPath for localization is not a string."
}
local function LogError(error, argument)
    error = error or 0
    argument = argument or ""
    local func, end_message = "unknown", ""
    if error >= 1 and error <= 3 then
        func = "AddMutator()"
        end_message = "Mutator not added!"
    elseif error >= 4 and error <= 8 then
        func = "AddCategory()"
        end_message = "Category not added!"
    elseif error >= 9 and error <= 12 then
        func = "AddTexture()"
        end_message = "Texture not added!"
    elseif error ~= 0 then
        func = "AddLocalization()"
        end_message = "Localization not added!"
    end
    log("[MutatorHelper]:" .. func .. " " .. Errors[error]:gsub("%s", argument or "") .. " " .. end_message)
end

--[[
    Adds a new mutator to the game

    Parameters:
        Mutator <- class
        -> Mutator Class

        MutatorPath <- string (optional)
        -> Path to the new mutator. Pass here 'ModPath' variable when you are calling this function.
    
    Returns:
        Nothing

    Exceptions:
        'Mutator' parameter is nil.
        'MutatorPath' parameter is not a string.
        Mutator has custom texture but 'path' variable is nil.

    Additional Notes:
        Do not call this function in the BLT Hook before you cache your 'ModPath' variable in your mod!

        Loaded textures should be 128x128 and saved as DDS format (A8R8G8B8).
        Also do not forget to change the extension to "texture".
]]
function MutatorHelper:AddMutator(Mutator, MutatorPath)
    if not Mutator then
        LogError(1)
        return
    end

    if MutatorPath then
        if type(MutatorPath) ~= "string" then
            LogError(2, tostring(Mutator))
            return
        end

        if Mutator.texture and Mutator.texture.custom then
            if not Mutator.texture.path then
                Log(3, tostring(Mutator))
                return
            end
            DB:create_entry(Idstring("texture"), Idstring(Mutator.texture.path), MutatorPath .. Mutator.texture.path .. ".texture")
        end

        local LocPath = MutatorPath .. "loc/" .. (Mutator.localization and (Mutator.localization.path .. "/") or "")

        if cached_language then
            if cached_language ~= "english" then
                table.insert(self.CustomMutatorsLanguage, LocPath .. cached_language .. ".json")
            end
            table.insert(self.CustomMutatorsLanguage, LocPath .. "english.json")
        else
            table.insert(self.CustomMutatorsLanguageDefer, LocPath)
        end
    end

    if type(Mutator.categories) == "table" then
        local category_to_remove = {}
        for _, category in ipairs(Mutator.categories) do
            if type(category) == "string" then
                local pos = self.CustomCategoryPos[category]
                if pos then
                    self.CustomCategories[pos].count = self.CustomCategories[pos].count + 1
                end
            else
                category_to_remove[category] = true
            end
        end
        if next(category_to_remove) then
            local new_categories = {}
            for _, category in ipairs(Mutator.categories) do
                if not category_to_remove[category] then
                    table.insert(new_categories, category)
                end
            end
            Mutator.categories = new_categories
        end
        if not table.contains(Mutator.categories, "custom_mutators") then
            table.insert(Mutator.categories, "custom_mutators")
        end
    else
        Mutator.categories = { "custom_mutators" }
    end
    self.CustomCategories[1].count = self.CustomCategories[1].count + 1

    table.insert(self.CustomMutators, Mutator)
end

--[[
    Adds a new category to the Mutator selection screen

    Parameters:
        CategoryName <- string
        -> Category Identifier

        Localization <- table
            Table Expects:
                'string' variable <- string
                -> Reuses loaded strings from the game

                or

                'path' variable <- string
                -> Pass here 'ModPath' variable

                Note: It is not recommended to have both fields in the table.
                      But one of them must be in the table.
            
    Returns:
        Nothing
    
    Exceptions:
        'CategoryName' parameter is nil or not a string.
        'Localization' parameter is nil or does not have the required fields from above.
        'CategoryName' is equal to 'all', 'enemies', 'gameplay' or the category already exists.
]]
function MutatorHelper:AddCategory(CategoryName, Localization)
    if not CategoryName then
        LogError(4)
        return
    end

    if type(CategoryName) ~= "string" then
        LogError(5)
        return
    end

    if not Localization then
        LogError(6)
        return
    end

    if not (Localization.string or Localization.path) then
        LogError(7)
        return
    end

    if CategoryName == "all" or CategoryName == "enemies" or CategoryName == "gameplay" or self.CustomCategoryPos[CategoryName] then
        LogError(8, CategoryName)
        return
    end

    local category = { name = CategoryName, count = 0 }

    if Localization.string then
        if managers.localization then
            LocalizationManager._custom_localizations["menu_mutators_category_" .. CategoryName] = managers.localization:text(Localization.string)
        else
            self.CustomCategoryString = self.CustomCategoryString or {}
            self.CustomCategoryString[CategoryName] = Localization.string
        end
    end

    table.insert(self.CustomCategories, category)
    self.CustomCategoryPos[CategoryName] = self.CustomCategoryLastPos
    self.CustomCategoryLastPos = self.CustomCategoryLastPos + 1

    -- Language loading stuff
    if not Localization.path then
        return
    end
    local LocPath = Localization.path .. "loc/"

    if cached_language then
        if cached_language ~= "english" then
            table.insert(self.CustomCategoriesLanguageFile, LocPath .. cached_language .. ".json")
        end
        table.insert(self.CustomCategoriesLanguageFile, LocPath .. "english.json")
    else
        table.insert(self.CustomMutatorsLanguageDefer, LocPath)
    end
end

--[[
    Adds a new texture for the mutators to use

    Parameters:
        TexturePath <- string
        -> Path to the texture (relative to your Mod folder path)
           Do not write the full path here e.q.:  "D:\SteamLibrary\steam\common\PAYDAY 2\mods\MyMutatorTemplate\guis\textures"
           Correct path e.q.: "guis/textures"
        
        ModPath <- string
        -> Pass here 'ModPath' variable

    Returns:
        Nothing
    
    Exceptions:
        'TexturePath' variable is nil or not a string
        'ModPath' variable is nil or not a string
    
    Additional Notes:
        'TexturePath' and 'ModPath' are not checked if they are correct.
        Make sure you pass the correct values.

        Loaded textures should be 128x128 (recommended) or have the same dimensions (eg. 64x64, 32x32, ...) and saved as DDS format (A8R8G8B8).
        Also do not forget to change the extension to "texture".
]]
function MutatorHelper:AddTexture(TexturePath, ModPath)
    if not TexturePath then
        LogError(9)
        return
    end

    if type(TexturePath) ~= "string" then
        LogError(10)
        return
    end

    if not ModPath then
        LogError(11)
        return
    end

    if type(ModPath) ~= "string" then
        LogError(12)
        return
    end

    DB:create_entry(Idstring("texture"), Idstring(TexturePath), ModPath .. TexturePath .. ".texture")
end

--[[
    Adds localization for mutators

    Parameters:
        ModPath <- string
        -> Pass here 'ModPath' variable
    
    Returns:
        Nothing

    Exceptions:
        'ModPath' variable is nil or not a string
]]
function MutatorHelper:AddLocalization(ModPath)
    if not ModPath then
        LogError(13)
        return
    end

    if type(ModPath) ~= "string" then
        LogError(14)
        return
    end

    local LocPath = ModPath .. "loc/"

    if cached_language then
        if cached_language ~= "english" then
            table.insert(self.CustomMutatorsLanguage, LocPath .. cached_language .. ".json")
        end
        table.insert(self.CustomMutatorsLanguage, LocPath .. "english.json")
    else
        table.insert(self.CustomMutatorsLanguageDefer, LocPath)
    end
end

local function CacheLanguage()
    if MutatorHelper.options.language == 1 then
        local LanguageFilename = nil
        local LanguageKey =
        {
            ["PAYDAY 2 THAI LANGUAGE Mod"] = "thai",
            ["Ultimate Localization Manager & 正體中文化"] = "tchinese",
            ["gabsF's Translate PT-BR Pack"] = "portuguese-br"
        }

        for _, mod in ipairs(BLT.Mods:Mods() or {}) do
            LanguageFilename = mod:IsEnabled() and LanguageKey[mod:GetName()] or nil
            if LanguageFilename then
                break
            end
        end

        local str
        for _, v in pairs(Language) do
            str = v
            if Idstring(str) and Idstring(str):key() == SystemInfo:language():key() then
                LanguageFilename = str
                break
            end
        end
        cached_language = LanguageFilename
    else
        cached_language = Language[MutatorHelper.options.language]
    end
end

local function Load()
    local file = io.open(MutatorHelper.SavePath, "r")
    if file then
        MutatorHelper.options.language = json.decode(file:read('*all'))["language"] or 1
        file:close()
    else
        MutatorHelper.options.language = 1
    end
end

Load()

Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_MutatorHelper", function(loc)
    CacheLanguage()
    -- Mutators Helper language loading code
    if cached_language ~= "english" then
        loc:load_localization_file(MutatorHelper.LocPath .. cached_language .. ".json")
    end
    loc:load_localization_file(MutatorHelper.LocPath .. "english.json", false)
    loc:load_localization_file(MutatorHelper.LocPath .. "languages.json")
    -- End

    -- Loading languages for Mutators
    for _, v in ipairs(MutatorHelper.CustomMutatorsLanguage) do
        loc:load_localization_file(v, not v:find("english.json"))
    end
    for _, v in ipairs(MutatorHelper.CustomCategoriesLanguageFile) do
        loc:load_localization_file(v, not v:find("english.json"))
    end
    for _, path in ipairs(MutatorHelper.CustomMutatorsLanguageDefer) do
        if cached_language ~= "english" then
            loc:load_localization_file(path .. cached_language .. ".json")
        end
        loc:load_localization_file(path .. "english.json", false)
    end
    for category, string in pairs(MutatorHelper.CustomCategoryString or {}) do
        LocalizationManager._custom_localizations["menu_mutators_category_" .. category] = loc:text(string)
    end
end)